@@ -36,7 +36,8 @@
 #include <stdlib.h>
 #include <stdio.h>
 
-#define SUFFIX ".tablet"
+#define TABLET_SUFFIX ".tablet"
+#define STYLUS_SUFFIX ".stylus"
 #define FEATURES_GROUP "Features"
 #define DEVICE_GROUP "Device"
 #define BUTTONS_GROUP "Buttons"
@@ -59,6 +60,14 @@ libwacom_class_string_to_enum(const char *class)
 		return WCLASS_BAMBOO;
 	if (strcmp(class, "Graphire") == 0)
 		return WCLASS_GRAPHIRE;
+	if (strcmp(class, "Intuos") == 0)
+		return WCLASS_INTUOS;
+	if (strcmp(class, "Intuos2") == 0)
+		return WCLASS_INTUOS2;
+	if (strcmp(class, "ISDV4") == 0)
+		return WCLASS_ISDV4;
+	if (strcmp(class, "PenDisplay") == 0)
+		return WCLASS_PEN_DISPLAYS;
 
 	return WCLASS_UNKNOWN;
 }
@@ -114,22 +123,52 @@ bus_to_str (WacomBusType bus)
 	g_assert_not_reached ();
 }
 
-static int
-libwacom_matchstr_to_ints(const char *match, uint32_t *vendor_id, uint32_t *product_id, WacomBusType *bus)
+char *
+make_match_string (WacomBusType bus, int vendor_id, int product_id)
 {
-	char busstr[64];
-	int rc;
+	return g_strdup_printf("%s:%04x:%04x", bus_to_str (bus), vendor_id, product_id);
+}
+
+static int
+libwacom_matchstr_to_matches(WacomDevice *device, const char *match)
+{
+	int rc = 1;
+	char **strs;
+	int i, nmatches = 0;
+	WacomBusType first_bus;
+	int first_vendor_id, first_product_id;
 
 	if (match == NULL)
 		return 0;
 
-	rc = sscanf(match, "%63[^:]:%x:%x", busstr, vendor_id, product_id);
-	if (rc != 3)
-		return 0;
+	strs = g_strsplit(match, ";", 0);
+	for (i = 0; strs[i] != NULL; i++) {
+		char busstr[64];
+		int vendor_id, product_id;
+		WacomBusType bus;
+		rc = sscanf(strs[i], "%63[^:]:%x:%x", busstr, &vendor_id, &product_id);
+		if (rc != 3) {
+			DBG("failed to match '%s' for product/vendor IDs. Skipping.\n", strs[i]);
+			continue;
+		}
+		bus = bus_from_str (busstr);
 
-	*bus = bus_from_str (busstr);
+		libwacom_update_match(device, bus, vendor_id, product_id);
 
-	return 1;
+		if (nmatches == 0) {
+			first_bus = bus;
+			first_vendor_id = vendor_id;
+			first_product_id = product_id;
+		}
+		nmatches++;
+	}
+
+	/* set default to first entry */
+	if (nmatches > 1)
+		libwacom_update_match(device, first_bus, first_vendor_id, first_product_id);
+
+	g_strfreev(strs);
+	return i;
 }
 
 static void
@@ -145,7 +184,6 @@ libwacom_parse_stylus_keyfile(WacomDeviceDatabase *db, const char *path)
 
 	rc = g_key_file_load_from_file(keyfile, path, G_KEY_FILE_NONE, &error);
 	g_assert (rc);
-
 	groups = g_key_file_get_groups (keyfile, NULL);
 	for (i = 0; groups[i]; i++) {
 		WacomStylus *stylus;
@@ -189,6 +227,8 @@ libwacom_parse_stylus_keyfile(WacomDeviceDatabase *db, const char *path)
 		g_hash_table_insert (db->stylus_ht, GINT_TO_POINTER (id), stylus);
 	}
 	g_strfreev (groups);
+	if (keyfile)
+		g_key_file_free (keyfile);
 }
 
 struct {
@@ -292,18 +332,17 @@ libwacom_parse_tablet_keyfile(const char *path)
 
 	match = g_key_file_get_string(keyfile, DEVICE_GROUP, "DeviceMatch", NULL);
 	if (g_strcmp0 (match, GENERIC_DEVICE_MATCH) == 0) {
-		device->match = match;
+		libwacom_update_match(device, WBUSTYPE_UNKNOWN, 0, 0);
 	} else {
-		if (!libwacom_matchstr_to_ints(match, &device->vendor_id, &device->product_id, &device->bus)) {
-			DBG("failed to match '%s' for product/vendor IDs in '%s'\n", device->match, path);
+		if (libwacom_matchstr_to_matches(device, match) == 0) {
+			DBG("failed to match '%s' for product/vendor IDs in '%s'\n", match, path);
 			g_free (match);
 			g_free (device);
 			device = NULL;
 			goto out;
 		}
-		device->match = g_strdup_printf ("%s:%04x:%04x", bus_to_str (device->bus), device->vendor_id, device->product_id);
-		g_free (match);
 	}
+	g_free (match);
 
 	device->name = g_key_file_get_string(keyfile, DEVICE_GROUP, "Name", NULL);
 	device->width = g_key_file_get_integer(keyfile, DEVICE_GROUP, "Width", NULL);
@@ -329,6 +368,10 @@ libwacom_parse_tablet_keyfile(const char *path)
 		}
 		g_strfreev (styli_list);
 		device->supported_styli = (int *) g_array_free (array, FALSE);
+	} else {
+		device->supported_styli = g_new (int, 1);
+		*device->supported_styli = 0xfffff;
+		device->num_styli = 1;
 	}
 
 	/* Features */
@@ -352,17 +395,17 @@ libwacom_parse_tablet_keyfile(const char *path)
 
 	if (device->features & FEATURE_BUILTIN &&
 	    device->features & FEATURE_REVERSIBLE)
-		g_warning ("Tablet '%s' is both reversible and builtin. This is impossible", device->match);
+		g_warning ("Tablet '%s' is both reversible and builtin. This is impossible", libwacom_get_match(device));
 
 	if (!(device->features & FEATURE_RING) &&
 	    (device->features & FEATURE_RING2))
-		g_warning ("Table '%s' has Ring2 but no Ring. This is impossible", device->match);
+		g_warning ("Table '%s' has Ring2 but no Ring. This is impossible", libwacom_get_match(device));
 
 	device->num_strips = g_key_file_get_integer(keyfile, FEATURES_GROUP, "NumStrips", NULL);
 	device->num_buttons = g_key_file_get_integer(keyfile, FEATURES_GROUP, "Buttons", &error);
 	if (device->num_buttons == 0 &&
 	    g_error_matches (error, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
-		g_warning ("Tablet '%s' has no buttons defined, do something!", device->match);
+		g_warning ("Tablet '%s' has no buttons defined, do something!", libwacom_get_match(device));
 		g_clear_error (&error);
 	}
 	if (device->num_buttons > 0) {
@@ -380,7 +423,7 @@ out:
 }
 
 static int
-scandir_filter(const struct dirent *entry)
+scandir_tablet_filter(const struct dirent *entry)
 {
 	const char *name = entry->d_name;
 	int len, suffix_len;
@@ -389,13 +432,31 @@ scandir_filter(const struct dirent *entry)
 		return 0;
 
 	len = strlen(name);
-	suffix_len = strlen(SUFFIX);
+	suffix_len = strlen(TABLET_SUFFIX);
 	if (len <= suffix_len)
 		return 0;
 
-	return !strcmp(&name[len - suffix_len], SUFFIX);
+	return !strcmp(&name[len - suffix_len], TABLET_SUFFIX);
 }
 
+static int
+scandir_stylus_filter(const struct dirent *entry)
+{
+	const char *name = entry->d_name;
+	int len, suffix_len;
+
+	if (!name || name[0] == '.')
+		return 0;
+
+	len = strlen(name);
+	suffix_len = strlen(STYLUS_SUFFIX);
+	if (len <= suffix_len)
+		return 0;
+
+	return !strcmp(&name[len - suffix_len], STYLUS_SUFFIX);
+}
+
+
 WacomDeviceDatabase *
 libwacom_database_new_for_path (const char *datadir)
 {
@@ -412,7 +473,7 @@ libwacom_database_new_for_path (const char *datadir)
 					   g_free,
 					   (GDestroyNotify) libwacom_destroy);
 
-    n = scandir(datadir, &files, scandir_filter, alphasort);
+    n = scandir(datadir, &files, scandir_tablet_filter, alphasort);
     if (n <= 0) {
 	    libwacom_database_destroy(db);
 	    return NULL;
@@ -421,13 +482,22 @@ libwacom_database_new_for_path (const char *datadir)
     nfiles = n;
     while(n--) {
 	    WacomDevice *d;
+	    const WacomMatch **matches, **match;
 
 	    path = g_build_filename (datadir, files[n]->d_name, NULL);
 	    d = libwacom_parse_tablet_keyfile(path);
 	    g_free(path);
 
-	    if (d)
-		    g_hash_table_insert (db->device_ht, g_strdup (d->match), d);
+	    if (!d)
+		    continue;
+
+	    matches = libwacom_get_matches(d);
+	    for (match = matches; *match; match++) {
+		    const char *matchstr;
+		    matchstr = libwacom_match_get_match_string(*match);
+		    g_hash_table_insert (db->device_ht, g_strdup (matchstr), d);
+		    d->refcnt++;
+	    }
     }
 
     while(nfiles--)
@@ -435,12 +505,26 @@ libwacom_database_new_for_path (const char *datadir)
     free(files);
 
     /* Load styli */
-    path = g_build_filename (datadir, STYLUS_DATA_FILE, NULL);
+    n = scandir(datadir, &files, scandir_stylus_filter, alphasort);
+    if (n <= 0) {
+	    libwacom_database_destroy(db);
+	    return NULL;
+    }
+
     db->stylus_ht = g_hash_table_new_full (g_direct_hash,
 					   g_direct_equal,
 					   NULL,
 					   (GDestroyNotify) libwacom_stylus_destroy);
-    libwacom_parse_stylus_keyfile(db, path);
+    nfiles = n;
+    while(n--) {
+	    path = g_build_filename (datadir, files[n]->d_name, NULL);
+	    libwacom_parse_stylus_keyfile(db, path);
+	    g_free(path);
+    }
+
+    while(nfiles--)
+	    free(files[nfiles]);
+    free(files);
 
     /* If we couldn't load _anything_ then something's wrong */
     if (g_hash_table_size (db->device_ht) == 0 &&
