@@ -29,9 +29,13 @@
 #endif
 
 
+/** @cond hide_from_doxygen */
 #ifndef _LIBWACOM_H_
 #define _LIBWACOM_H_
+/** @endcond */
 
+#include <stdint.h>
+#include <stdio.h>
 /**
  @mainpage
 
@@ -45,16 +49,21 @@
  The usage of libwacom in an application could look like this:
 
  <pre>
+      WacomDeviceDatabase *db;
       WacomDevice *device;
+      WacomError *error;
 
-      device = libwacom_new_from_path("/dev/input/event0");
+      db = libwacom_database_new();
+      error = libwacom_error_new();
+      device = libwacom_new_from_path(db, "/dev/input/event0", WFALLBACK_NONE, error);
       if (!device)
            return; // should check for error here
 
       if (libwacom_device_is_builtin(device))
            printf("This is a built-in device\n");
 
-      libwacom_destroy(&device);
+      libwacom_destroy(device);
+      libwacom_database_destroy(db);
  </pre>
 
  For a full API reference to see libwacom.h.
@@ -72,6 +81,8 @@
 
 typedef struct _WacomDevice WacomDevice;
 
+typedef struct _WacomMatch WacomMatch;
+
 typedef struct _WacomStylus WacomStylus;
 
 typedef struct _WacomError WacomError;
@@ -85,49 +96,53 @@ typedef struct _WacomDeviceDatabase WacomDeviceDatabase;
  * Possible error codes.
  */
 enum WacomErrorCode {
-    WERROR_NONE,		/**< No error has occured */
-    WERROR_BAD_ALLOC,		/**< Allocation error */
-    WERROR_INVALID_PATH,	/**< A path specified is invalid */
-    WERROR_INVALID_DB,		/**< The passed DB is invalid */
-    WERROR_BAD_ACCESS,		/**< Invalid permissions to access the path */
-    WERROR_UNKNOWN_MODEL,	/**< Unsupported/unknown device */
+	WERROR_NONE,		/**< No error has occured */
+	WERROR_BAD_ALLOC,		/**< Allocation error */
+	WERROR_INVALID_PATH,	/**< A path specified is invalid */
+	WERROR_INVALID_DB,		/**< The passed DB is invalid */
+	WERROR_BAD_ACCESS,		/**< Invalid permissions to access the path */
+	WERROR_UNKNOWN_MODEL,	/**< Unsupported/unknown device */
 };
 
 /**
  * Bus types for tablets.
  */
 typedef enum {
-    WBUSTYPE_UNKNOWN,		/**< Unknown/unsupported bus type */
-    WBUSTYPE_USB,		/**< USB tablet */
-    WBUSTYPE_SERIAL,		/**< Serial tablet */
-    WBUSTYPE_BLUETOOTH		/**< Bluetooth tablet */
+	WBUSTYPE_UNKNOWN,		/**< Unknown/unsupported bus type */
+	WBUSTYPE_USB,		/**< USB tablet */
+	WBUSTYPE_SERIAL,		/**< Serial tablet */
+	WBUSTYPE_BLUETOOTH		/**< Bluetooth tablet */
 } WacomBusType;
 
 /**
  * Classes of devices.
  */
 typedef enum {
-    WCLASS_UNKNOWN,		/**< Unknown/unsupported device class */
-    WCLASS_INTUOS3,		/**< Any Intuos3 series */
-    WCLASS_INTUOS4,		/**< Any Intuos4 series */
-    WCLASS_CINTIQ,		/**< Any Cintiq device */
-    WCLASS_BAMBOO,		/**< Any Bamboo device */
-    WCLASS_GRAPHIRE,		/**< Any Graphire device */
-    WCLASS_ISDV4,		/**< Any serial ISDV4 device */
+	WCLASS_UNKNOWN,		/**< Unknown/unsupported device class */
+	WCLASS_INTUOS3,		/**< Any Intuos3 series */
+	WCLASS_INTUOS4,		/**< Any Intuos4 series */
+	WCLASS_INTUOS5,		/**< Any Intuos5 series */
+	WCLASS_CINTIQ,		/**< Any Cintiq device */
+	WCLASS_BAMBOO,		/**< Any Bamboo device */
+	WCLASS_GRAPHIRE,	/**< Any Graphire device */
+	WCLASS_ISDV4,		/**< Any serial ISDV4 device */
+	WCLASS_INTUOS,		/**< Any Intuos series */
+	WCLASS_INTUOS2,		/**< Any Intuos2 series */
+	WCLASS_PEN_DISPLAYS,	/**< Any "interactive pen display" */
 } WacomClass;
 
 /**
  * Class of stylus
  */
 typedef enum {
-    WSTYLUS_UNKNOWN,
-    WSTYLUS_GENERAL,
-    WSTYLUS_INKING,
-    WSTYLUS_AIRBRUSH,
-    WSTYLUS_CLASSIC,
-    WSTYLUS_MARKER,
-    WSTYLUS_STROKE,
-    WSTYLUS_PUCK
+	WSTYLUS_UNKNOWN,
+	WSTYLUS_GENERAL,
+	WSTYLUS_INKING,
+	WSTYLUS_AIRBRUSH,
+	WSTYLUS_CLASSIC,
+	WSTYLUS_MARKER,
+	WSTYLUS_STROKE,
+	WSTYLUS_PUCK
 } WacomStylusType;
 
 /**
@@ -150,6 +165,16 @@ typedef enum {
 	WACOM_BUTTON_TOUCHSTRIPS_MODESWITCH = (WACOM_BUTTON_TOUCHSTRIP_MODESWITCH | WACOM_BUTTON_TOUCHSTRIP2_MODESWITCH),
 } WacomButtonFlags;
 
+typedef enum {
+	WFALLBACK_NONE = 0,
+	WFALLBACK_GENERIC = 1
+} WacomFallbackFlags;
+
+typedef enum {
+	WCOMPARE_NORMAL		= 0,		/**< compare the device only */
+	WCOMPARE_MATCHES	= (1 << 1),	/**< compare all possible matches too */
+} WacomCompareFlags;
+
 /**
  * Allocate a new structure for error reporting.
  *
@@ -196,10 +221,10 @@ WacomDeviceDatabase* libwacom_database_new(void);
 WacomDeviceDatabase* libwacom_database_new_for_path(const char *datadir);
 
 /**
-  * Free all memory used by the database.
-  *
-  * @param db A Tablet and Stylus database.
-  */
+ * Free all memory used by the database.
+ *
+ * @param db A Tablet and Stylus database.
+ */
 void libwacom_database_destroy(WacomDeviceDatabase *db);
 
 /**
@@ -214,7 +239,7 @@ void libwacom_database_destroy(WacomDeviceDatabase *db);
  *
  * @return A new reference to this device or NULL on errror.
  */
-WacomDevice* libwacom_new_from_path(WacomDeviceDatabase *db, const char *path, int fallback, WacomError *error);
+WacomDevice* libwacom_new_from_path(WacomDeviceDatabase *db, const char *path, WacomFallbackFlags fallback, WacomError *error);
 
 /**
  * Create a new device reference from the given vendor/product IDs.
@@ -243,6 +268,28 @@ WacomDevice* libwacom_new_from_usbid(WacomDeviceDatabase *db, int vendor_id, int
  */
 WacomDevice* libwacom_new_from_name(WacomDeviceDatabase *db, const char *name, WacomError *error);
 
+/**
+ * Returns the list of devices in the given database.
+ *
+ * @param db A device database
+ * @param error If not NULL, set to the error if any occurs
+ *
+ * @return A NULL terminated list of pointers to all the devices inside the
+ * database.
+ * The content of the list is owned by the database and should not be
+ * modified of freed. Use free() to free the list.
+ */
+WacomDevice** libwacom_list_devices_from_database(WacomDeviceDatabase *db, WacomError *error);
+
+/**
+ * Print the description of this device to the given file.
+ *
+ * @param fd The file descriptor to print to
+ * @param device The device to print the description for.
+ */
+void libwacom_print_device_description (int fd, WacomDevice *device);
+
+
 /**
  * Remove the device and free all memory and references to it.
  *
@@ -250,6 +297,17 @@ WacomDevice* libwacom_new_from_name(WacomDeviceDatabase *db, const char *name, W
  */
 void libwacom_destroy(WacomDevice *device);
 
+/**
+ * Compare the two devices for equal-ness.
+ *
+ * @param a The first device
+ * @param b The second device
+ * @param flags Flags to dictate what constitutes a match
+ *
+ * @return 0 if the devices are identical, nonzero otherwise
+ */
+int libwacom_compare(WacomDevice *a, WacomDevice *b, WacomCompareFlags flags);
+
 /**
  * @param device The tablet to query
  * @return The class of the device
@@ -270,10 +328,18 @@ int libwacom_get_vendor_id(WacomDevice *device);
 
 /**
  * @param device The tablet to query
- * @return The first match for the device in question
+ * @return The current match string used for this device (if set) or the first
+ * match string in the tablet definition.
  */
 const char* libwacom_get_match(WacomDevice *device);
 
+/**
+ * @param device The tablet to query
+ * @return A pointer to the null-terminated list of possible matches for this device. Do not
+ * modify this pointer or any content!
+ */
+const WacomMatch** libwacom_get_matches(WacomDevice *device);
+
 /**
  * @param device The tablet to query
  * @return The numeric product ID for this device
@@ -313,6 +379,8 @@ int libwacom_has_stylus(WacomDevice *device);
 int libwacom_has_touch(WacomDevice *device);
 
 /**
+ * Tablet buttons are numbered 'A' through to 'A' + number of buttons.
+ *
  * @param device The tablet to query
  * @return The number of buttons on the tablet
  */
@@ -323,7 +391,7 @@ int libwacom_get_num_buttons(WacomDevice *device);
  * @param num_styli Return location for the number of listed styli
  * @return an array of Styli IDs supported by the device
  */
-int *libwacom_get_supported_styli(WacomDevice *device, int *num_styli);
+const int *libwacom_get_supported_styli(WacomDevice *device, int *num_styli);
 
 /**
  * @param device The tablet to query
@@ -441,6 +509,19 @@ int         libwacom_stylus_has_lens (const WacomStylus *stylus);
  */
 WacomStylusType libwacom_stylus_get_type (const WacomStylus *stylus);
 
+/**
+ * Print the description of this stylus to the given file.
+ *
+ * @param fd The file descriptor
+ * @param stylus The stylus to print the description for.
+ */
+void libwacom_print_stylus_description (int fd, const WacomStylus *stylus);
+
+WacomBusType libwacom_match_get_bustype(const WacomMatch *match);
+uint32_t libwacom_match_get_product_id(const WacomMatch *match);
+uint32_t libwacom_match_get_vendor_id(const WacomMatch *match);
+const char* libwacom_match_get_match_string(const WacomMatch *match);
+
 #endif /* _LIBWACOM_H_ */
 
-/* vim: set noexpandtab shiftwidth=8: */
+/* vim: set noexpandtab tabstop=8 shiftwidth=8: */
