@@ -39,7 +39,6 @@
 	printf(__VA_ARGS__)
 
 #define GENERIC_DEVICE_MATCH "generic"
-#define STYLUS_DATA_FILE "libwacom.stylus"
 
 typedef enum {
 	IS_BUILTIN_UNSET	= -1,
@@ -52,28 +51,46 @@ enum WacomFeature {
 	FEATURE_TOUCH		= (1 << 1),
 	FEATURE_RING		= (1 << 2),
 	FEATURE_RING2		= (1 << 3),
-	FEATURE_VSTRIP		= (1 << 4),
-	FEATURE_HSTRIP		= (1 << 5),
-	FEATURE_BUILTIN		= (1 << 6),
-	FEATURE_REVERSIBLE	= (1 << 7)
+	FEATURE_BUILTIN		= (1 << 4),
+	FEATURE_REVERSIBLE	= (1 << 5)
 };
 
+/* WARNING: When adding new members to this struct
+ * make sure to update libwacom_copy_match() ! */
+struct _WacomMatch {
+	char *match;
+	WacomBusType bus;
+	uint32_t vendor_id;
+	uint32_t product_id;
+};
+
+/* WARNING: When adding new members to this struct
+ * make sure to update libwacom_copy() and
+ * libwacom_print_device_description() ! */
 struct _WacomDevice {
-	char *vendor;
-	char *product;
+	char *name;
 	int width;
 	int height;
 
-	char *match;
-	uint32_t vendor_id;
-	uint32_t product_id;
+	int match;	/* used match or first match by default */
+	WacomMatch **matches; /* NULL-terminated */
+	int nmatches; /* not counting NULL-terminated element */
 
 	WacomClass cls;
-	WacomBusType bus;
-	int num_buttons;
-	int *supported_styli;
-	gsize num_styli;
+	int num_strips;
 	uint32_t features;
+
+	int strips_num_modes;
+	int ring_num_modes;
+	int ring2_num_modes;
+
+	gsize num_styli;
+	int *supported_styli;
+
+	int num_buttons;
+	WacomButtonFlags *buttons;
+
+	gint refcnt; /* for the db hashtable */
 };
 
 struct _WacomStylus {
@@ -82,6 +99,7 @@ struct _WacomStylus {
 	int num_buttons;
 	gboolean has_eraser;
 	gboolean is_eraser;
+	gboolean has_lens;
 	WacomStylusType type;
 };
 
@@ -98,9 +116,12 @@ struct _WacomError {
 /* INTERNAL */
 void libwacom_error_set(WacomError *error, enum WacomErrorCode code, const char *msg, ...);
 void libwacom_stylus_destroy(WacomStylus *stylus);
+void libwacom_update_match(WacomDevice *device, WacomBusType bus, int vendor_id, int product_id);
 
 WacomBusType  bus_from_str (const char *str);
 const char   *bus_to_str   (WacomBusType bus);
+char *make_match_string(WacomBusType bus, int vendor_id, int product_id);
+
 
 #endif /* _LIBWACOMINT_H_ */
 
